package dev.fitko.fitconnect.api.services.events;

import dev.fitko.fitconnect.api.domain.model.destination.Destination;
import dev.fitko.fitconnect.api.domain.model.event.Event;
import dev.fitko.fitconnect.api.domain.model.event.EventLog;
import dev.fitko.fitconnect.api.domain.model.event.EventLogEntry;
import dev.fitko.fitconnect.api.domain.model.event.SubmissionStatus;
import dev.fitko.fitconnect.api.domain.model.event.authtags.AuthenticationTags;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;
import dev.fitko.fitconnect.api.exceptions.internal.EventLogException;

import java.util.List;
import java.util.UUID;

/**
 * A service that provides access to the {@link Submission}s {@link EventLog} via the FIT-Connect REST-API.
 * Every event is a recorded step within the process of sending a submission or a case.
 * Events are encoded as security event tokens (SET).
 *
 * @see <a href="https://docs.fitko.de/fit-connect/docs/getting-started/event-log/events">Event Documentation</a>
 */
public interface EventLogService {

    /**
     * Get the {@link EventLog} by caseId.
     *
     * @param caseId unique case identifier
     * @param destinationId unique identifier of the destination
     * @return list of {@link EventLogEntry}
     * @throws EventLogException if a technical error occurred or the validation failed
     */
    List<EventLogEntry> getEventLog(UUID caseId, UUID destinationId) throws EventLogException;

    /**
     * Retrieve the current status of a {@link Submission}.
     *
     * @param destinationId unique identifier of the {@link Destination} the log should be retrieved for
     * @param caseId unique identifier of the case the log should be retrieved for
     * @param submissionId unique identifier of the submission the log should be retrieved for
     * @param authenticationTags {@link AuthenticationTags} used for SET-Event integrity validation
     *
     * @return {@link SubmissionStatus} the current status
     * @throws EventLogException if a technical error occurred or the validation failed
     */
    SubmissionStatus getLastedEvent(UUID destinationId, UUID caseId, UUID submissionId, AuthenticationTags authenticationTags) throws EventLogException;

    /**
     * Retrieve the current status of a {@link Submission} with skipping the authentication tag validation.
     *
     * @param destinationId unique identifier of the {@link Destination} the log should be retrieved for
     * @param caseId unique identifier of the case the log should be retrieved for
     * @param submissionId unique identifier of the submission the log should be retrieved for
     *
     * @return {@link SubmissionStatus} the current status
     * @throws EventLogException if a technical error occurred or the validation failed
     */
    SubmissionStatus getLastedEventWithoutAuthTagValidation(UUID destinationId, UUID caseId, UUID submissionId) throws EventLogException;

    /**
     * Send an event for a given caseId.
     *
     * @param caseId unique case identifier
     * @param signedAndSerializedSET the serialised and signed event as SET string
     * @throws EventLogException if a technical error occurred
     */
    void sendEvent(UUID caseId, String signedAndSerializedSET) throws EventLogException;

    /**
     * Get authentication tags for a given event and caseId.
     *
     * @param event the event type to filter the log for
     * @param submission submission data
     * @return {@link AuthenticationTags} for metadata, data and attachments
     * @throws EventLogException if a technical error occurred or the validation failed
     */
    AuthenticationTags getAuthenticationTagsForEvent(Event event, Submission submission) throws EventLogException;
}
